/*
 * This file is part of system-settings
 *
 * Copyright (C) 2013 Canonical Ltd.
 *
 * Contact: Didier Roche <didier.roches@canonical.com>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtSystemInfo 5.0
import SystemSettings 1.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import Ubuntu.SystemSettings.Update 1.0
import "js/utils.js" as Utils


ItemPage {
    id: root

    title: i18n.tr("Updates")

    DeviceInfo {
        id: deviceInfo
    }

    Component {
         id: dialogInstallComponent
         Dialog {
             id: dialogueInstall
             title: i18n.tr("Update System")
             text: i18n.tr("The phone needs to restart to install the system update.")

             Button {
                 text: i18n.tr("Install & Restart")
                 color: UbuntuColors.orange
                 onClicked: {
                     updateManager.applySystemUpdate();
                     PopupUtils.close(dialogueInstall);
                     installingImageUpdate.visible = true;
                 }
             }
             Button {
                 text: i18n.tr("Not Now")
                 color: UbuntuColors.greyGradient
                 onClicked: {
                      PopupUtils.close(dialogueInstall);
                 }
             }
         }
    }

    property int updates: 0

    //states
    states: [
        State {
            name: "SEARCHING"
            PropertyChanges { target: notification; visible: false}
            PropertyChanges { target: installAllButton; visible: false}
            PropertyChanges { target: checkForUpdatesArea; visible: true}
            PropertyChanges { target: checkForUpdatesButton; visible: false}
        },
        State {
            name: "NOUPDATES"
            PropertyChanges { target: notification; text: i18n.tr("No updates found")}
            PropertyChanges { target: notification; onClicked: undefined }
            PropertyChanges { target: notification; progression: false}
            PropertyChanges { target: notification; visible: true}
            PropertyChanges { target: checkForUpdatesButton; visible: true}
            PropertyChanges { target: installAllButton; visible: false}
            PropertyChanges { target: checkForUpdatesArea; visible: true}
        },
        State {
            name: "UPDATES"
            PropertyChanges { target: notification; visible: false}
            PropertyChanges { target: checkForUpdatesArea; visible: false}
            PropertyChanges { target: checkForUpdatesButton; visible: false}
            PropertyChanges { target: installAllButton; visible: true}
        },
        State {
            name: "1UPDATE"
            PropertyChanges { target: notification; visible: false}
            PropertyChanges { target: installAllButton; visible: false}
            PropertyChanges { target: checkForUpdatesArea; visible: false}
            PropertyChanges { target: checkForUpdatesButton; visible: true}
        },
        State {
            name: "NOCREDENTIALS"
            PropertyChanges { target: notification; text: i18n.tr("Please log into your Ubuntu One account in <a href='settings:///system/online-accounts'>System Settings</a>.")}
            PropertyChanges { target: notification; onClicked: root.open_online_accounts() }
            PropertyChanges { target: notification; progression: true}
            PropertyChanges { target: notification; visible: true}
            PropertyChanges { target: installAllButton; visible: false}
            PropertyChanges { target: checkForUpdatesButton; visible: false}
            PropertyChanges { target: checkForUpdatesArea; visible: true}
        }
    ]

    function open_online_accounts() {
        Qt.openUrlExternally("settings:///system/online-accounts");
    }

    onUpdatesChanged: {
        if(root.updates > 1) {
            root.state = "UPDATES";
        } else {
            root.state = "1UPDATE";
        }
    }

    UpdateManager {
        id: updateManager
        objectName: "updateManager"

        Component.onCompleted: {
            root.state = "SEARCHING";
            updateManager.checkUpdates();
        }

        onUpdateAvailableFound: {
            root.updates = updateManager.model.length;
        }

        onUpdatesNotFound: {
            root.state = "NOUPDATES";
        }

        onCredentialsNotFound: {
            root.state = "NOCREDENTIALS";
        }

        onSystemUpdateDownloaded: {
            PopupUtils.open(dialogNoCards);
        }
    }

    Item {
        id: checkForUpdatesArea
        objectName: "checkForUpdatesArea"
        visible: root.updates == 0 ? true : false
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            topMargin: units.gu(2)
            leftMargin: units.gu(4)
            rightMargin: units.gu(4)
        }
        height: buttonCheckUpdates.height

        ActivityIndicator {
            id: activity
            running: checkForUpdatesArea.visible
            visible: activity.running
            anchors {
                left: parent.left
                top: parent.top
                topMargin: units.gu(1)
            }
        }

        Label {
            text: i18n.tr("Checking for updates...")
            fontSize: "large"
            verticalAlignment: parent.verticalCenter
            elide: Text.ElideRight
            anchors {
                left: activity.right
                top: parent.top
                right: buttonCheckUpdates.left
                leftMargin: units.gu(2)
                topMargin: units.gu(1)
                rightMargin: units.gu(2)
            }
        }

        Button {
            id: buttonCheckUpdates
            text: i18n.tr("Retry")
            anchors {
                right: parent.right
                top: parent.top
            }

            onClicked: {
                root.state = "SEARCHING";
                updateManager.checkUpdates();
            }
        }
    }

    Button {
        id: checkForUpdatesButton
        objectName: "checkForUpdatesButton"
        text: i18n.tr("Check for updates")
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            topMargin: units.gu(2)
            leftMargin: units.gu(4)
            rightMargin: units.gu(4)
        }
        visible: false

        color: UbuntuColors.orange
        onClicked: {
            root.state = "SEARCHING";
            updateManager.checkUpdates();
        }
    }

    Button {
        id: installAllButton
        objectName: "installAllButton"
        text: i18n.tr("Install %1 Updates").arg(root.updates)
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            topMargin: units.gu(2)
            leftMargin: units.gu(4)
            rightMargin: units.gu(4)
        }
        visible: false

        color: UbuntuColors.orange
        onClicked: {
            for(var i=0; i < updateManager.model.length; i++){
                var item = updateManager.model[i];
                if(!item.selected) {
                    updateManager.startDownload(item.packageName, item.systemUpdate);
                }
            }
        }
    }

    ListView {
        id: updateList
        objectName: "updateList"
        anchors {
            left: parent.left
            right: parent.right
            top: installAllButton.bottom
            bottom: notification.visible ? notification.top : configuration.top
            margins: units.gu(2)
            bottomMargin: 0
        }
        model: updateManager.model
        clip: true

        delegate: ListItem.Subtitled {
            id: listItem
            icon: Qt.resolvedUrl(modelData.iconUrl)
            enabled: !modelData.updateState
            height: units.gu(9)

            Rectangle {
                id: textArea
                objectName: "textArea"
                color: "transparent"
                anchors.fill: parent
                anchors.topMargin: units.gu(1)

                property string message: modelData.error

                onMessageChanged: {
                    if(message.length > 0) {
                        labelVersion.text = message;
                        buttonAppUpdate.text = i18n.tr("Retry");
                        modelData.updateState = false;
                    }
                }

                Column {
                    id: colRight
                    objectName: "colRight"
                    anchors.top: parent.top
                    anchors.right: parent.right
                    anchors.topMargin: units.gu(1)

                    Button {
                        id: buttonAppUpdate
                        text: i18n.tr("Update")
                        height: textArea.height / 2

                        onClicked: {
                            modelData.selected = true;
                            updateManager.startDownload(modelData.packageName, modelData.systemUpdate);
                        }
                    }
                    Label {
                        text: Utils.convert_bytes_to_size(modelData.binaryFilesize)
                        height: textArea.height / 2
                        anchors.right: parent.right
                    }
                }

                Column {
                    id: colLeft
                    objectName: "colLeft"
                    anchors {
                        left: parent.left
                        bottom: parent.bottom
                        top: parent.top
                        right: colRight.left
                        rightMargin: units.gu(1)
                        topMargin: units.gu(1)
                    }

                    Label {
                        anchors {
                            left: parent.left
                            right: parent.right
                        }
                        text: modelData.title
                        color: modelData.updateState ? "gray" : "black"
                        font.bold: true
                        elide: Text.ElideRight
                    }
                    Label {
                        id: labelVersion
                        anchors {
                            left: parent.left
                            right: parent.right
                        }
                        text: modelData.remoteVersion ? i18n.tr("Version: ") + modelData.remoteVersion : ""
                        color: "black"
                        visible: !modelData.updateState
                        elide: Text.ElideRight
                    }
                }
            }

            ProgressBar {
                id: progress
                objectName: "progress"
                width: textArea.width - colRight.width - units.gu(2)
                height: parent.height / 3
                anchors.left: textArea.left
                anchors.leftMargin: units.gu(1)
                anchors.bottom: textArea.bottom
                anchors.bottomMargin: units.dp(5)
                opacity: modelData.updateState ? 1 : 0
                value: 0
                minimumValue: 0
                maximumValue: 100

                DownloadTracker {
                    id: tracker
                    objectName: "tracker"
                    service: "com.canonical.applications.Downloader"
                    dbusPath: modelData.updateState ? modelData.dbusPath : ""

                    onProgress: {
                        var percentage = parseInt(received * 100 / total);
                        progress.value = percentage;
                    }

                    onFinished: {
                        progress.visible = false;
                        buttonAppUpdate.visible = false;
                        textArea.message = i18n.tr("Installed");
                        root.updates -= 1;
                    }

                    onError: {
                        textArea.message = error;
                    }
                }

                Behavior on opacity { PropertyAnimation { duration: UbuntuAnimation.SleepyDuration } }
            }
        }
    }

    ListItem.Standard {
        id: notification
        anchors.bottom: configuration.top
    }

    ListItem.SingleValue {
        id: configuration
        anchors.bottom: parent.bottom
        text: i18n.tr("Auto download")
        value: {
            if (updateManager.downloadMode === 0)
                return i18n.tr("Never")
            else if (updateManager.downloadMode === 1)
                return i18n.tr("On wi-fi")
            else if (updateManager.downloadMode === 2)
                return i18n.tr("Always")
        }
       progression: true
       onClicked: pageStack.push(Qt.resolvedUrl("Configuration.qml"), {updateBackend: updateManager})
    }

    Rectangle {
        id: installingImageUpdate
        anchors.fill: parent
        visible: false

        color: UbuntuColors.midAubergine

        Column {
            anchors.centerIn: parent

            Image {
                source: Qt.resolvedUrl("file:///usr/share/ubuntu/settings/system/icons/distributor-logo.png")
                anchors.horizontalCenter: parent.horizontalCenter
            }

            ProgressBar {
                indeterminate: true
                anchors.horizontalCenter: parent.horizontalCenter
            }

            Label {
                text: i18n.tr("Installing update...")
                anchors.horizontalCenter: parent.horizontalCenter
            }
        }
    }

}
