/*
 * Copyright 2013 Canonical Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "update_manager.h"
#include <QString>
#include <QStringList>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonValue>
#include <QDebug>

namespace UpdatePlugin {

UpdateManager::UpdateManager(QObject *parent):
    QObject(parent),
    m_process(this)
{
    // SSO SERVICE
    QObject::connect(&m_service, SIGNAL(credentialsFound(const Token&)),
                     this, SLOT(handleCredentialsFound(Token)));
    QObject::connect(&m_service, SIGNAL(credentialsNotFound()),
                     this, SLOT(handleCredentialsNotFound()));
    // PROCESS
    QObject::connect(&(m_process), SIGNAL(finished(int)),
                  this, SLOT(processOutput()));
    // NETWORK
    QObject::connect(&(m_network), SIGNAL(updatesFound()),
                  this, SLOT(processUpdates()));
    QObject::connect(&(m_network), SIGNAL(updatesNotFound()),
                  this, SIGNAL(updatesNotFound()));
    QObject::connect(&(m_network), SIGNAL(errorOccurred()),
                  this, SIGNAL(errorFound()));
    QObject::connect(&(m_network),
                     SIGNAL(clickTokenObtained(Update*, const QString&)),
                     this, SLOT(clickTokenReceived(Update*, const QString&)));
    QObject::connect(&(m_network),
                     SIGNAL(downloadUrlFound(const QString&, const QString&)),
                     this,
                     SLOT(downloadUrlObtained(const QString&, const QString&)));
    // DOWNLOADER
    QObject::connect(&(m_downloader),
                     SIGNAL(downloadCreated(const QString&, const QString&)),
                     this,
                     SLOT(downloadCreated(const QString&, const QString&)));
    QObject::connect(&(m_downloader),
                     SIGNAL(downloadNotCreated(const QString&, const QString&)),
                     this,
                     SLOT(downloadNotCreated(const QString&, const QString&)));
    // SYSTEM UPDATE
    QObject::connect(&m_systemUpdate, SIGNAL(updateAvailable(Update*)),
                  this, SLOT(registerSystemUpdate(Update*)));
    QObject::connect(&m_systemUpdate, SIGNAL(downloadModeChanged()),
                  SIGNAL(downloadModeChanged()));
    QObject::connect(&m_systemUpdate, SIGNAL(updateDownloaded()),
                  SIGNAL(systemUpdateDownloaded()));
}

UpdateManager::~UpdateManager()
{
}

void UpdateManager::handleCredentialsFound(Token token)
{
    qDebug() << "Credentials found.";
    m_token = token;
    QStringList args("list");
    args << "--manifest";
    m_process.start("click", args);
}

void UpdateManager::handleCredentialsNotFound()
{
    qDebug() << "No credentials were found.";
    Q_EMIT credentialsNotFound();
}

void UpdateManager::checkUpdates()
{
    m_model.clear();
    Q_EMIT modelChanged();
    m_systemUpdate.checkForUpdate();
    m_service.getCredentials();
}

void UpdateManager::processOutput()
{
    QString output(m_process.readAllStandardOutput());

    QJsonDocument document = QJsonDocument::fromJson(output.toUtf8());

    QJsonArray array = document.array();

    int i;
    for (i = 0; i < array.size(); i++) {
        QJsonObject object = array.at(i).toObject();
        QString name = object.value("name").toString();
        QString title = object.value("title").toString();
        QString version = object.value("version").toString();
        Update *app = new Update();
        app->initializeApplication(name, title, version);
        m_apps[app->getPackageName()] = app;
    }

    m_network.checkForNewVersions(m_apps);
}

void UpdateManager::processUpdates()
{
    foreach (QString id, m_apps.keys()) {
        Update *app = m_apps.value(id);
        if(app->updateRequired()) {
            m_model.append(QVariant::fromValue(app));
        }
    }

    Q_EMIT modelChanged();
    Q_EMIT updateAvailableFound();
}

void UpdateManager::registerSystemUpdate(Update *update)
{
    m_model.insert(0, QVariant::fromValue(update));
    Q_EMIT modelChanged();
    Q_EMIT updateAvailableFound();
}

void UpdateManager::startDownload(const QString &packagename, bool systemUpdate)
{
    qDebug() << "Download Package:" << packagename;
    if (systemUpdate) {
        m_systemUpdate.downloadUpdate();
    } else {
        m_network.getResourceUrl(packagename);
    }
}

void UpdateManager::downloadUrlObtained(const QString &packagename,
                                        const QString &url)
{
    if (m_token.isValid()) {
        QString authHeader = m_token.signUrl(url, QStringLiteral("HEAD"), true);
        qDebug() << "Download Url:" << url;
        Update *app = m_apps[packagename];
        app->setClickUrl(url);
        m_network.getClickToken(app, url, authHeader);
    } else {
        Update *app = m_apps[packagename];
        app->setError("Invalid User Token");
    }
}

void UpdateManager::downloadCreated(const QString &packagename,
                                    const QString &dbuspath)
{
    qDebug() << "Dbus Path:" << dbuspath;
    m_apps[packagename]->setDbusPath(dbuspath);
    m_apps[packagename]->setUpdateState(true);
}

void UpdateManager::downloadNotCreated(const QString &packagename,
                                       const QString &error)
{
    qDebug() << "Download not created";
    Update *app = m_apps[packagename];
    app->setError(error);
}

void UpdateManager::clickTokenReceived(Update *app, const QString &clickToken)
{
    app->setError("");
    m_downloader.startDownload(app->getPackageName(),
                               app->getClickUrl(), clickToken);
}

}
