/*
 * Copyright 2013 Canonical Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <QTest>

#include "update_manager.h"
#include "update.h"

using namespace UpdatePlugin;

class UpdateManagerTest : public QObject
{
    Q_OBJECT

public slots:
    void slotCallback();

private slots:
    void cleanup();

    void testCheckUpdatesModelSignal();
    void testCheckUpdatesUpdateSignal();
    void testStartDownload();
    void testTokenObtained();
    void testDownloadNotCreated();

private:
    int m_signal_counter;
};

void UpdateManagerTest::cleanup()
{
    this->m_signal_counter = 0;
}

void UpdateManagerTest::slotCallback()
{
    this->m_signal_counter++;
}

void UpdateManagerTest::testCheckUpdatesModelSignal()
{
    this->m_signal_counter = 0;
    UpdateManager manager;
    this->connect(&(manager), SIGNAL(modelChanged()),
                  this, SLOT(slotCallback()));
    QCOMPARE(manager.get_apps().size(), 0);
    manager.checkUpdates();
    QCOMPARE(manager.get_apps().size(), 4);
    QCOMPARE(manager.get_model().size(), 1);
    Update* app = manager.get_model()[0].value<Update*>();
    QCOMPARE(app->getTitle(), QString("XDA Developers App"));
    QCOMPARE(app->updateRequired(), true);
    QCOMPARE(app->getPackageName(), QString("com.ubuntu.developer.xda-app"));
    QCOMPARE(this->m_signal_counter, 2);
}

void UpdateManagerTest::testCheckUpdatesUpdateSignal()
{
    this->m_signal_counter = 0;
    UpdateManager manager;
    this->connect(&(manager), SIGNAL(updateAvailableFound()),
                  this, SLOT(slotCallback()));
    QCOMPARE(manager.get_apps().size(), 0);
    manager.checkUpdates();
    QCOMPARE(manager.get_apps().size(), 4);
    QCOMPARE(manager.get_model().size(), 1);
    Update* app = manager.get_model()[0].value<Update*>();
    QCOMPARE(app->getTitle(), QString("XDA Developers App"));
    QCOMPARE(app->updateRequired(), true);
    QCOMPARE(app->getPackageName(), QString("com.ubuntu.developer.xda-app"));
    QCOMPARE(this->m_signal_counter, 1);
}

void UpdateManagerTest::testStartDownload()
{
    UpdateManager manager;
    manager.checkUpdates();
    Update* app = manager.get_model()[0].value<Update*>();
    manager.startDownload(app->getPackageName(), false);
    QCOMPARE(app->updateState(), true);
    QCOMPARE(app->dbusPath(), QString("/com/canonical/download/path"));
}

void UpdateManagerTest::testTokenObtained()
{
    UpdateManager manager;
    manager.checkUpdates();
    QCOMPARE(manager.get_token().isValid(), true);
}

void UpdateManagerTest::testDownloadNotCreated()
{
    UpdateManager manager;
    manager.checkUpdates();
    Token t;
    manager.set_token(t);
    Update* app = manager.get_model()[0].value<Update*>();
    manager.startDownload(app->getPackageName(), false);
    QCOMPARE(app->updateState(), false);
    QCOMPARE(app->dbusPath(), QString(""));
    QCOMPARE(app->getError(), QString("Invalid User Token"));
}

QTEST_MAIN(UpdateManagerTest)
#include "tst_updatemanager.moc"
